@@ -454,7 +454,8 @@ setMethod(
             selcov <- lower.tri(sig, diag=TRUE)
             scores2 <- matrix(0, nrow(tabdata), sum(selcov))
             thetas2 <- numeric(sum(selcov))
-            ret <- .Call('EAPgroup', log(itemtrace), tabdata, Theta, prior, mu)
+            ret <- .Call('EAPgroup', log(itemtrace), tabdata, Theta, prior, mu, 
+                         mirtClusterEnv$ncores)
             tmp <- cbind(ret$scores, ret$scores2) * r
             newpars <- apply(tmp, 2, sum) / N
             if(nfact > 1L){
