@@ -1,4 +1,4 @@
-EM.group <- function(pars, constrain, PrepList, list, Theta, DERIV)
+EM.group <- function(pars, constrain, Ls, PrepList, list, Theta, DERIV)
 {
     verbose <- list$verbose
     nfact <- list$nfact
@@ -6,8 +6,8 @@ EM.group <- function(pars, constrain, PrepList, list, Theta, DERIV)
         stop('empirical histogram only available for unidimensional models')
     NCYCLES <- list$NCYCLES
     TOL <- list$TOL
-    MSTEPTOL <- list$MSTEPTOL
     BFACTOR <- list$BFACTOR
+    CUSTOM.IND <- list$CUSTOM.IND
     itemloc <- list$itemloc
     ngroups <- length(pars)
     specific <- list$specific
@@ -17,11 +17,7 @@ EM.group <- function(pars, constrain, PrepList, list, Theta, DERIV)
     nfullpars <- 0L
     estpars <- c()
     prodlist <- PrepList[[1L]]$prodlist
-    gfulldata <- gtheta0 <- gstructgrouppars <- vector('list', ngroups)
     for(g in 1L:ngroups){
-        gstructgrouppars[[g]] <- ExtractGroupPars(pars[[g]][[J+1L]])
-        gfulldata[[g]] <- PrepList[[g]]$fulldata
-        gtheta0[[g]] <- matrix(0, nrow(gfulldata[[g]]), nfact)
         for(i in 1L:(J+1L)){
             nfullpars <- nfullpars + length(pars[[g]][[i]]@par)
             estpars <- c(estpars, pars[[g]][[i]]@est)
@@ -45,24 +41,10 @@ EM.group <- function(pars, constrain, PrepList, list, Theta, DERIV)
             ind1 <- ind2 + 1L
         }
     }
-    stagecycle <- 1L
     converge <- 1L
-    inverse_fail_count <- 1L
-    ##
-    L <- c()
-    for(g in 1L:ngroups)
-        for(i in 1L:(J+1L))
-            L <- c(L, pars[[g]][[i]]@est)
     estindex <- index[estpars]
-    L <- diag(as.numeric(L))
-    redun_constr <- rep(FALSE, length(estpars))
-    if(length(constrain) > 0L){
-        for(i in 1L:length(constrain)){
-            L[constrain[[i]], constrain[[i]]] <- 1L/length(constrain[[i]])
-            for(j in 2L:length(constrain[[i]]))
-                redun_constr[constrain[[i]][j]] <- TRUE
-        }
-    }
+    L <- Ls$L; L2 <- Ls$L2; L3 <- Ls$L3
+    redun_constr <- Ls$redun_constr
     estindex_unique <- index[estpars & !redun_constr]
     if(any(diag(L)[!estpars] > 0L)){
         redindex <- index[!estpars]
@@ -70,7 +52,7 @@ EM.group <- function(pars, constrain, PrepList, list, Theta, DERIV)
              paste(paste0(redindex[diag(L)[!estpars] > 0L]), ''), ' but should only be applied to
                  estimated parameters. Please fix!')
     }
-    Prior <- prior <- Priorbetween <- gstructgrouppars <- rlist <- r <- list()
+    prior <- rlist <- r <- list()
     #make sure constrained pars are equal
     tmp <- L
     tmp2 <- diag(tmp)
@@ -103,19 +85,20 @@ EM.group <- function(pars, constrain, PrepList, list, Theta, DERIV)
     names(longpars) <- names(est)
     EMhistory <- matrix(NA, NCYCLES+1L, length(longpars))
     EMhistory[1L,] <- longpars
-    gTheta <- vector('list', ngroups)
     ANY.PRIOR <- rep(FALSE, ngroups)
-    NO.CUSTOM <- !any(sapply(pars[[1L]], class) %in% 'custom')
-    for(g in 1L:ngroups){
-        gTheta[[g]] <- Theta
-        if(length(prodlist) > 0L)
-            gTheta[[g]] <- prodterms(gTheta[[g]],prodlist)
-        if(BFACTOR){
-            Thetabetween <- thetaComb(theta=theta, nfact=nfact-ncol(sitems))
+    if(length(prodlist) > 0L)
+        Theta <- prodterms(Theta, prodlist)
+    if(BFACTOR){
+        Thetabetween <- thetaComb(theta=theta, nfact=nfact-ncol(sitems))
+        for(g in 1L:ngroups){
             prior[[g]] <- dnorm(theta, 0, 1)
             prior[[g]] <- prior[[g]]/sum(prior[[g]])
         }
+    }
+    gTheta <- vector('list', ngroups)
+    for(g in 1L:ngroups){
         ANY.PRIOR[g] <- any(sapply(pars[[g]], function(x) x@any.prior))
+        gTheta[[g]] <- Theta
     }
     preMstep.longpars2 <- preMstep.longpars <- longpars
     accel <- 0
@@ -126,39 +109,22 @@ EM.group <- function(pars, constrain, PrepList, list, Theta, DERIV)
     for (cycles in 1L:NCYCLES){
         #priors
         start <- proc.time()[3L]
-        for(g in 1L:ngroups){
-            gstructgrouppars[[g]] <- ExtractGroupPars(pars[[g]][[J+1L]])
-            if(BFACTOR){
-                sel <- 1L:(nfact-ncol(sitems))
-                Priorbetween[[g]] <- mvtnorm::dmvnorm(Thetabetween,
-                                        gstructgrouppars[[g]]$gmeans[sel],
-                                        gstructgrouppars[[g]]$gcov[sel,sel,drop=FALSE])
-                Priorbetween[[g]] <- Priorbetween[[g]]/sum(Priorbetween[[g]])
-                Prior[[g]] <- apply(expand.grid(Priorbetween[[g]], prior[[g]]), 1, prod)
-                next
-            }
-            Prior[[g]] <- mvtnorm::dmvnorm(gTheta[[g]][ ,1L:nfact,drop=FALSE],gstructgrouppars[[g]]$gmeans,
-                                           gstructgrouppars[[g]]$gcov)
-            Prior[[g]] <- Prior[[g]]/sum(Prior[[g]])
-        }
-        if(list$EH && cycles > 1L){
-            for(g in 1L:ngroups)
-                Prior[[g]] <- rowSums(rlist[[g]][[1L]]) / sum(rlist[[g]][[1L]])
-        } else if(!is.null(list$customPriorFun)){
-            for(g in 1L:ngroups)
-                Prior[[g]] <- list$customPriorFun(gTheta[[g]])
-        }
+        tmp <- updatePrior(pars=pars, Theta=Theta, Thetabetween=Thetabetween,
+                           list=list, ngroups=ngroups, nfact=nfact, prior=prior,
+                           J=J, BFACTOR=BFACTOR, sitems=sitems, cycles=cycles, rlist=rlist)
+        Prior <- tmp$Prior; Priorbetween <- tmp$Priorbetween
         #Estep
         LL <- 0
         for(g in 1L:ngroups){
             if(BFACTOR){
                 rlist[[g]] <- Estep.bfactor(pars=pars[[g]], tabdata=PrepList[[g]]$tabdata,
-                                            Theta=gTheta[[g]], prior=prior[[g]], Prior=Prior[[g]],
-                                            Priorbetween=Priorbetween[[g]], specific=specific, sitems=sitems,
-                                            itemloc=itemloc, NO.CUSTOM=NO.CUSTOM)
+                                            Theta=Theta, prior=prior[[g]], Prior=Prior[[g]],
+                                            Priorbetween=Priorbetween[[g]], specific=specific, 
+                                            sitems=sitems, itemloc=itemloc, CUSTOM.IND=CUSTOM.IND)
             } else {
-                rlist[[g]] <- Estep.mirt(pars=pars[[g]], tabdata=PrepList[[g]]$tabdata, NO.CUSTOM=NO.CUSTOM,
-                                         Theta=gTheta[[g]], prior=Prior[[g]], itemloc=itemloc)
+                rlist[[g]] <- Estep.mirt(pars=pars[[g]], tabdata=PrepList[[g]]$tabdata, 
+                                         CUSTOM.IND=CUSTOM.IND, Theta=Theta, 
+                                         prior=Prior[[g]], itemloc=itemloc)
             }
             LL <- LL + sum(r[[g]]*log(rlist[[g]]$expected))
         }
@@ -176,7 +142,8 @@ EM.group <- function(pars, constrain, PrepList, list, Theta, DERIV)
         if(all(!est) && !groupest) break
         longpars <- Mstep(pars=pars, est=est, longpars=longpars, ngroups=ngroups, J=J,
                           gTheta=gTheta, itemloc=itemloc, Prior=Prior, ANY.PRIOR=ANY.PRIOR,
-                          NO.CUSTOM=NO.CUSTOM, PrepList=PrepList, L=L, UBOUND=UBOUND, LBOUND=LBOUND,
+                          CUSTOM.IND=CUSTOM.IND, SLOW.IND=list$SLOW.IND, groupest=groupest, 
+                          PrepList=PrepList, L=L, UBOUND=UBOUND, LBOUND=LBOUND, 
                           rlist=rlist, constrain=constrain, cycle=cycles, DERIV=DERIV)
         if(list$accelerate && cycles > 10L && cycles %% 3 == 0L){
             dX2 <- preMstep.longpars - preMstep.longpars2
@@ -207,23 +174,22 @@ EM.group <- function(pars, constrain, PrepList, list, Theta, DERIV)
         ind1 <- 1L
         for(group in 1L:ngroups){
             for (i in 1L:J){
-                deriv <- Deriv(x=pars[[group]][[i]], Theta=gTheta[[g]], EM = TRUE,
-                               estHess=TRUE)
+                deriv <- Deriv(x=pars[[group]][[i]], Theta=Theta, estHess=TRUE)
                 ind2 <- ind1 + length(deriv$grad) - 1L
                 h[ind1:ind2, ind1:ind2] <- pars[[group]][[i]]@hessian <- deriv$hess
                 ind1 <- ind2 + 1L
             }
             i <- i + 1L
-            deriv <- Deriv(x=pars[[group]][[i]], Theta=gTheta[[g]], EM = TRUE,
+            deriv <- Deriv(x=pars[[group]][[i]], CUSTOM.IND=CUSTOM.IND,
+                           Theta=Theta, EM = TRUE,
                            pars=pars[[group]], tabdata=PrepList[[group]]$tabdata,
                            itemloc=itemloc, estHess=TRUE)
             ind2 <- ind1 + length(deriv$grad) - 1L
             h[ind1:ind2, ind1:ind2] <- pars[[group]][[i]]@hessian <- deriv$hess
             ind1 <- ind2 + 1L
         }
-        hess <- L %*% h %*% L
+        hess <- updateHess(h=h, L2=L2, L3=L3)
         hess <- hess[estpars & !redun_constr, estpars & !redun_constr]
-        #return internal functions for SEM.SE
         return(list(pars=pars, cycles = cycles, info=matrix(0), longpars=longpars, converge=converge,
                     logLik=LL, rlist=rlist, SElogLik=0, L=L, infological=infological,
                     estindex_unique=estindex_unique, correction=correction, hess=hess, Prior=Prior,
@@ -234,12 +200,13 @@ EM.group <- function(pars, constrain, PrepList, list, Theta, DERIV)
     ret <- list(pars=pars, cycles = cycles, info=matrix(0), longpars=longpars, converge=converge,
                 logLik=LL, rlist=rlist, SElogLik=0, L=L, infological=infological,
                 estindex_unique=estindex_unique, correction=correction, hess=hess, random=list(),
-                Prior=Prior, time=c(Estep=as.numeric(Estep.time), Mstep=as.numeric(Mstep.time)))
+                Prior=Prior, time=c(Estep=as.numeric(Estep.time), Mstep=as.numeric(Mstep.time)),
+                prior=prior, Priorbetween=Priorbetween, sitems=sitems)
     ret
 }
 
 # Estep for mirt
-Estep.mirt <- function(pars, tabdata, Theta, prior, itemloc, NO.CUSTOM=FALSE,
+Estep.mirt <- function(pars, tabdata, Theta, prior, itemloc, CUSTOM.IND,
                        itemtrace=NULL, deriv = FALSE)
 {
     nquad <- nrow(Theta)
@@ -247,45 +214,42 @@ Estep.mirt <- function(pars, tabdata, Theta, prior, itemloc, NO.CUSTOM=FALSE,
     r <- tabdata[ ,ncol(tabdata)]
     X <- tabdata[ ,1L:(ncol(tabdata) - 1L), drop = FALSE]
     if(is.null(itemtrace))
-        itemtrace <- computeItemtrace(pars=pars, Theta=Theta, itemloc=itemloc, NO.CUSTOM=NO.CUSTOM)
-    retlist <- .Call("Estep", itemtrace, prior, X, r)
+        itemtrace <- computeItemtrace(pars=pars, Theta=Theta, itemloc=itemloc, CUSTOM.IND=CUSTOM.IND)
+    retlist <- .Call("Estep", itemtrace, prior, X, r, mirtClusterEnv$ncores)
     if(deriv) retlist$itemtrace <- itemtrace
     return(retlist)
 }
 
 # Estep for bfactor
 Estep.bfactor <- function(pars, tabdata, Theta, prior, Prior, Priorbetween, specific,
-                          NO.CUSTOM=FALSE, sitems, itemloc, itemtrace=NULL)
+                          CUSTOM.IND, sitems, itemloc, itemtrace=NULL)
 {
     J <- length(itemloc) - 1L
     r <- tabdata[ ,ncol(tabdata)]
-    X <- tabdata[ ,1L:(ncol(tabdata) - 1L)]
+    X <- tabdata[ ,1L:(ncol(tabdata) - 1L), drop = FALSE]
     if(is.null(itemtrace))
-        itemtrace <- computeItemtrace(pars=pars, Theta=Theta, itemloc=itemloc, NO.CUSTOM=NO.CUSTOM)
-    retlist <- .Call("Estepbfactor", itemtrace, prior, Priorbetween, X, r, sitems)
-    r1 <- matrix(0, nrow(Theta), ncol(X))
-    for (i in 1L:J){
-        r1[ ,itemloc[i]:(itemloc[i+1L]-1L)] <-
-            retlist$r1[ ,itemloc[i]:(itemloc[i+1L]-1L) + (specific[i] - 1L)*ncol(X) ]
-    }
-    r1 <- r1 * Prior
-    return(list(r1=r1, expected=retlist$expected))
+        itemtrace <- computeItemtrace(pars=pars, Theta=Theta, itemloc=itemloc, CUSTOM.IND=CUSTOM.IND)
+    retlist <- .Call("Estepbfactor", itemtrace, prior, Priorbetween, X, r, sitems, Prior,
+                     mirtClusterEnv$ncores)
+    return(retlist)
 }
 
 Mstep <- function(pars, est, longpars, ngroups, J, gTheta, itemloc, PrepList, L, ANY.PRIOR,
-                  UBOUND, LBOUND, constrain, cycle, DERIV, Prior, rlist, NO.CUSTOM){
+                  UBOUND, LBOUND, constrain, cycle, DERIV, Prior, rlist, CUSTOM.IND, 
+                  SLOW.IND, groupest){
     p <- longpars[est]
+    maxit <- ifelse(cycle > 10L, 25L, 10L)
     opt <- try(optim(p, fn=Mstep.LL, gr=Mstep.grad, method='L-BFGS-B',
-                     control=list(maxit=ifelse(cycle > 10L, 10L, 5L), fnscale = -1L),
-                     DERIV=DERIV, rlist=rlist, NO.CUSTOM=NO.CUSTOM,
+                     control=list(maxit=maxit, fnscale = -1L),
+                     DERIV=DERIV, rlist=rlist, CUSTOM.IND=CUSTOM.IND, SLOW.IND=SLOW.IND,
                      est=est, longpars=longpars, pars=pars, ngroups=ngroups, J=J, gTheta=gTheta,
                      PrepList=PrepList, L=L, constrain=constrain, ANY.PRIOR=ANY.PRIOR,
                      UBOUND=UBOUND, LBOUND=LBOUND, itemloc=itemloc, lower=LBOUND[est], upper=UBOUND[est]),
             silent=TRUE)
 #     #uncomment for testing with nlm
 #     opt <- try(optim(f=Mstep.LL, p,
-#                      DERIV=DERIV, rlist=rlist, NO.CUSTOM=NO.CUSTOM,
-#                      est=est, longpars=longpars, pars=pars, ngroups=ngroups, J=J, gTheta=gTheta,
+#                      DERIV=DERIV, rlist=rlist, CUSTOM.IND=CUSTOM.IND, SLOW.IND=SLOW.IND,
+#                      est=est, longpars=longpars, pars=pars, ngroups=ngroups, J=J, Theta=Theta,
 #                      PrepList=PrepList, L=L, constrain=constrain, ANY.PRIOR=ANY.PRIOR,
 #                      UBOUND=UBOUND, LBOUND=LBOUND, itemloc=itemloc),
 #                silent=TRUE)
@@ -296,19 +260,21 @@ Mstep <- function(pars, est, longpars, ngroups, J, gTheta, itemloc, PrepList, L,
         for(i in 1L:length(constrain))
             longpars[constrain[[i]][-1L]] <- longpars[constrain[[i]][1L]]
     i = J + 1L
-    for(group in 1L:ngroups){
-        if(any(pars[[group]][[i]]@est)){
-            newpars <- Deriv(x=pars[[group]][[i]], Theta=gTheta[[group]], EM = TRUE,
-                           pars=pars[[group]], tabdata=PrepList[[group]]$tabdata,
-                           itemloc=itemloc, prior=Prior[[group]])
-            longpars[pars[[group]][[i]]@parnum[pars[[group]][[i]]@est]] <- newpars
+    if(groupest){
+        for(group in 1L:ngroups){
+            if(any(pars[[group]][[i]]@est)){
+                newpars <- Deriv(x=pars[[group]][[i]], Theta=gTheta[[group]], EM = TRUE,
+                               pars=pars[[group]], tabdata=PrepList[[group]]$tabdata,
+                               itemloc=itemloc, prior=Prior[[group]], CUSTOM.IND=CUSTOM.IND)
+                longpars[pars[[group]][[i]]@parnum[pars[[group]][[i]]@est]] <- newpars
+            }
         }
     }
     return(longpars)
 }
 
-Mstep.LL <- function(p, est, longpars, pars, ngroups, J, gTheta, PrepList, L, NO.CUSTOM,
-                     constrain, LBOUND, UBOUND, itemloc, DERIV, rlist, ANY.PRIOR){
+Mstep.LL <- function(p, est, longpars, pars, ngroups, J, gTheta, PrepList, L, CUSTOM.IND, 
+                     SLOW.IND, constrain, LBOUND, UBOUND, itemloc, DERIV, rlist, ANY.PRIOR){
     longpars[est] <- p
     if(length(constrain) > 0L)
        for(i in 1L:length(constrain))
@@ -317,13 +283,13 @@ Mstep.LL <- function(p, est, longpars, pars, ngroups, J, gTheta, PrepList, L, NO
     LLs <- numeric(ngroups)
     for(g in 1L:ngroups)
         LLs[g] <- LogLikMstep(pars[[g]], Theta=gTheta[[g]], rs=rlist[[g]],
-                              itemloc=itemloc, NO.CUSTOM=NO.CUSTOM, any.prior=ANY.PRIOR[g])
-#     return(-sum(LLs))
+                              itemloc=itemloc, CUSTOM.IND=CUSTOM.IND, any.prior=ANY.PRIOR[g])
     return(sum(LLs))
 }
 
-LogLikMstep <- function(x, Theta, itemloc, rs, any.prior, NO.CUSTOM){
-    log_itemtrace <- log(computeItemtrace(pars=x, Theta=Theta, itemloc=itemloc, NO.CUSTOM=NO.CUSTOM))
+LogLikMstep <- function(x, Theta, itemloc, rs, any.prior, CUSTOM.IND){
+    log_itemtrace <- log(computeItemtrace(pars=x, Theta=Theta, 
+                                          itemloc=itemloc, CUSTOM.IND=CUSTOM.IND))
     LL <- sum(rs$r1 * log_itemtrace)
     if(any.prior){
         for(i in 1L:(length(x)-1L))
@@ -333,18 +299,21 @@ LogLikMstep <- function(x, Theta, itemloc, rs, any.prior, NO.CUSTOM){
     return(LL)
 }
 
-Mstep.grad <- function(p, est, longpars, pars, ngroups, J, gTheta, PrepList, L,  ANY.PRIOR,
-                       constrain, LBOUND, UBOUND, itemloc, DERIV, rlist, NO.CUSTOM){
+Mstep.grad <- function(p, est, longpars, pars, ngroups, J, gTheta, PrepList, L, ANY.PRIOR,
+                       constrain, LBOUND, UBOUND, itemloc, DERIV, rlist, CUSTOM.IND, 
+                       SLOW.IND){
     longpars[est] <- p
     if(length(constrain) > 0L)
         for(i in 1L:length(constrain))
             longpars[constrain[[i]][-1L]] <- longpars[constrain[[i]][1L]]
     pars <- reloadPars(longpars=longpars, pars=pars, ngroups=ngroups, J=J)
-    g <- rep(0, ncol(L))
-    for(group in 1L:ngroups){
-        for (i in 1L:J){
-            deriv <- DERIV[[group]][[i]](x=pars[[group]][[i]], Theta=gTheta[[group]], EM=TRUE)
-            g[pars[[group]][[i]]@parnum] <- deriv$grad
+    g <- .Call('computeDPars', pars, gTheta, matrix(0L, 1L, J), length(est), 0L, 0L)$grad
+    if(length(SLOW.IND)){        
+        for(group in 1L:ngroups){
+            for (i in SLOW.IND){
+                deriv <- DERIV[[group]][[i]](x=pars[[group]][[i]], Theta=gTheta[[group]])
+                g[pars[[group]][[i]]@parnum] <- deriv$grad
+            }
         }
     }
     grad <- g %*% L
